-- Jarcast Nan Madol Functions
-- Author: Jarcast
--==========================================================================================================================
-- INCLUDES
--==========================================================================================================================
include("FLuaVector.lua")
include("IconSupport.lua")
include("InstanceManager.lua")
include("PlotIterators.lua")
include("Cunt15_IteratorPack_v3.lua")
--==========================================================================================================================
-- GLOBALS
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
local Teams 					= Teams
local activePlayerID			= Game.GetActivePlayer()
local activePlayer				= Players[activePlayerID]
local activeTeamID				= activePlayer:GetTeam()
local activeTeam				= Teams[activeTeamID]
--==========================================================================================================================
-- UTILITIES
--==========================================================================================================================
-- UTILITIES
----------------------------------------------------------------------------------------------------------------------------
--Game_IsUsingMod
local CBPmodID = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
local CBOmodID = "8411a7a8-dad3-4622-a18e-fcc18324c799"
local CSDmodID = "eead0050-1e3f-4178-a91f-26cf1881ac39"
local EEraModID = "ce8aa614-7ef7-4a45-a179-5329869e8d6d"
local EECBPmodID  = "87b2deae-83cb-9b88-9017-79e741d3739c"

function Game_IsUsingMod(modID)
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == modID then
			return true
		end
	end
	return false
end

local isUsingEEra = Game_IsUsingMod(EEraModID) or Game_IsUsingMod(EECBPmodID)
local isUsingCBP  = Game_IsUsingMod(CBPmodID)
local isUsingCBO  = Game_IsUsingMod(CBOmodID)
----------------------------------------------------------------------------------------------------------------------------
--HasTrait
function HasTrait(player, traitID)
	if Game_IsUsingMod(CBPmodID) then 
		return player:HasTrait(traitID)
	else
		local leaderType = GameInfo.Leaders[player:GetLeaderType()].Type
		local traitType  = GameInfo.Traits[traitID].Type
		for row in GameInfo.Leader_Traits("LeaderType = '" .. leaderType .. "' AND TraitType = '" .. traitType .. "'") do
			return true
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
-- GetRandom
function GetRandom(lower, upper)
    return Game.Rand((upper + 1) - lower, "") + lower
end
-------------------------------------------------------------------------------------------------------------------------
--IsTraitActive
local slotComputerID = SlotStatus["SS_COMPUTER"]
local slotTakenID = SlotStatus["SS_TAKEN"]
function IsTraitActive(traitID)
	for slotID = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotPlayer = Players[slotID]
		local slotStatus = PreGame.GetSlotStatus(slotID)
		if (slotStatus == slotTakenID or slotStatus == slotComputerID) then
			if HasTrait(slotPlayer, traitID) then
				return true
			end
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	--Game.GetUserSetting
	function Game.GetUserSetting(type)
		for row in GameInfo.JFD_GlobalUserSettings("Type = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function IsInteger(n)
  return n==math.floor(n)
end
-------------------------------------------------------------------------------------------------------------------------
function Neirai_GetNearestCity(pPlayer, pPlot)
    local distance = 9999
    local cNearestCity = nil
    for cCity in pPlayer:Cities() do
        local pCityPlot = cCity:Plot()
        local between = Map.PlotDistance(pCityPlot:GetX(), pCityPlot:GetY(), pPlot:GetX(), pPlot:GetY())
        if between < distance then
            distance = between
            cNearestCity = cCity
        end
    end
    return cNearestCity
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	function Game.GetCustomModOptions(type)
		for row in GameInfo.CustomModOptions("Name = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	function Game.GetCommunityOptions(type)
		for row in GameInfo.Community("Type = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function GetImprovementIDFromResourceID( resourceID)
	tResImprovements  = {}
	for row in DB.Query("SELECT a.ID ResID, b.ResourceType ResType, c.ID ImprovID FROM Resources a, Improvement_ResourceTypes b, Improvements c WHERE a.Type = b.ResourceType and b.ImprovementType = c.Type and a.ID = '" .. resourceID .. "'") do
		table.insert(tResImprovements, row.ImprovID);
	end
	return tResImprovements[1]
end
--==========================================================================================================================
-- UNIQUE FUNCTIONS
--==========================================================================================================================
-- GLOBALS
--------------------------------------------------------------------------------------------------------------------------
local onlycoastal= GameDefines.LAKE_MAX_AREA_SIZE +1
local CityWorkingRadius = GameDefines.MAXIMUM_WORK_PLOT_DISTANCE or 3
local iMod = ((GameInfo.GameSpeeds[Game.GetGameSpeedType()].BuildPercent)/100)
local iCiv = GameInfoTypes.CIVILIZATION_JAR_NANMADOL
local traitNanMadolID = GameInfoTypes.TRAIT_JAR_NANMADOL
local g_IsTraitActive = IsTraitActive(traitNanMadolID)
local resCoralID = GameInfoTypes.RESOURCE_CORAL
local fAtollID = GameInfoTypes.FEATURE_ATOLL
local iCoast = GameInfoTypes.TERRAIN_COAST
local iKepidau = GameInfoTypes.BUILDING_JAR_KEPIDAU
local iKepidauAirlift = GameInfoTypes.BUILDING_JAR_GAKEPDUMMY
local iNahnken = GameInfoTypes.UNIT_JAR_NAHNKEN
local iNahnkenPromo = GameInfoTypes.PROMOTION_JAR_NAHNKEN
local iKepidauPromo = GameInfoTypes.PROMOTION_JAR_KEPIDAU_MOVE
local iKepidauPromo2 = GameInfoTypes.PROMOTION_JAR_KEPIDAU_MOVE2
local iKepidauPromo3 = GameInfoTypes.PROMOTION_JAR_KEPIDAU_MOVE3
local missionGAClaimSeaLux = GameInfoTypes.MISSION_JAR_GA_CLAIMSEALUX
local iGAClaimSeaLuxPromo = GameInfoTypes.PROMOTION_JAR_GACLAIMSEALUX
local CUSTOM_MISSION_NO_ACTION		 	= 0
local CUSTOM_MISSION_ACTION			 	= 1
local CUSTOM_MISSION_DONE            	= 2
local CUSTOM_MISSION_ACTION_AND_DONE 	= 3
------------------------------------------------------------------------------------------------------------------------
-- QUICK MOD BALANCING PANEL
------------------------------------------------------------------------------------------------------------------------
-- Here are the changeable values that determine the strength of the effects described in the design.
-- UA
local FactorProdFoodClaimedSeaLux = 2
local GAMissionPlotRadiusCheckedByAI = 2
-- UU
local FactorProductionFromNearbyKill = 0.75
local NahnkenGAPfactor = 7
-- UB
local bKepidauAirliftEnabled = (Game.GetCommunityOptions("JAR_NANMADOL_KEPIDAU_AIRLIFT") == 1)
-- UU2 

--------------------------------------------------------------------------------------------------------------------------
-- FUNCTIONS
--------------------------------------------------------------------------------------------------------------------------
-- NAN MADOL UA
--========================================================================================================================
function Jar_DefenseFromCoral(iPlayer, iUnitID)
	local pPlayer = Players[iPlayer]
	local pUnit = pPlayer:GetUnitByID(iUnitID)	
	local pPlot = pUnit:GetPlot()
	if not pPlot then return end
	if not HasTrait(pPlayer, traitNanMadolID) then return end
	local bcheck = (pPlot:GetResourceType() == resCoralID)
	pUnit:SetHasPromotion(GameInfoTypes.PROMOTION_JAR_NANMADOL_CORAL, bcheck)
end
if g_IsTraitActive then GameEvents.UnitSetXY.Add(Jar_DefenseFromCoral) end

local tFeatures= {}
for row in DB.Query("select ID from Features where Type in ('FEATURE_ATOLL','FEATURE_REEF','FEATURE_NEW_REEF_A','FEATURE_NEW_REEF_B','FEATURE_NEW_REEF_C')") do
	tFeatures[row.ID] = true
end

function ClaimAdjacentCoralAtoll(plot, iPlayer, cityID)
	if not plot then return end
	if plot:GetResourceType() == resCoralID or tFeatures[plot:GetFeatureType()] then
		for pAdjacentPlot in Cunt15_AdjacentPlotIterator(plot) do
			if pAdjacentPlot and pAdjacentPlot:GetOwner() ~= iPlayer then
				pAdjacentPlot:SetOwner(iPlayer, cityID, true, true);
				pAdjacentPlot:SetRevealed(Players[iPlayer]:GetTeam(), true)
				pAdjacentPlot:UpdateFog();
				local hexpos = ToHexFromGrid(Vector2(pAdjacentPlot:GetX(), pAdjacentPlot:GetY()));
				Events.HexFOWStateChanged(hexpos, true, false);
				if pAdjacentPlot:GetResourceType() == resCoralID or tFeatures[pAdjacentPlot:GetFeatureType()] then
					for zAdjacentPlot in Cunt15_AdjacentPlotIterator(pAdjacentPlot) do
						local distance = Map.PlotDistance(plot:GetX(), plot:GetY(), zAdjacentPlot:GetX(), zAdjacentPlot:GetY())
						if zAdjacentPlot and zAdjacentPlot:GetOwner() ~= iPlayer then
							zAdjacentPlot:SetOwner(iPlayer, cityID, true, true);
							zAdjacentPlot:SetRevealed(Players[iPlayer]:GetTeam(), true)
							zAdjacentPlot:UpdateFog();
							local hexpos2 = ToHexFromGrid(Vector2(zAdjacentPlot:GetX(), zAdjacentPlot:GetY()));
							Events.HexFOWStateChanged(hexpos2, true, false);
							if zAdjacentPlot:GetResourceType() == resCoralID or tFeatures[zAdjacentPlot:GetFeatureType()] then
								for wAdjacentPlot in Cunt15_AdjacentPlotIterator(zAdjacentPlot) do
									if wAdjacentPlot and wAdjacentPlot:GetOwner() ~= iPlayer then
										wAdjacentPlot:SetOwner(iPlayer, cityID, true, true);
										wAdjacentPlot:SetRevealed(Players[iPlayer]:GetTeam(), true)
										wAdjacentPlot:UpdateFog();
										local hexpos3 = ToHexFromGrid(Vector2(wAdjacentPlot:GetX(), wAdjacentPlot:GetY()));
										Events.HexFOWStateChanged(hexpos3, true, false);
									end
								end									
							end
						end
					end
				end
			end
		end
	end
end

function Jar_NanMadol_AtollCoralCultureBomb(iPlayer, cityID, iX, iY, bGold, bFaithOrCulture)
	local pPlayer = Players[iPlayer]
	local pCity = pPlayer:GetCityByID(cityID)
	local pPlot = Map.GetPlot( iX, iY )
	if HasTrait(pPlayer, traitNanMadolID) and pPlayer:IsAlive() then
		ClaimAdjacentCoralAtoll(pPlot, iPlayer, cityID)
	end
end
if g_IsTraitActive then GameEvents.CityBoughtPlot.Add(Jar_NanMadol_AtollCoralCultureBomb) end

function Jar_NanMadol_AtollCoralCultureBomb2(iPlayer, iCityX, iCityY)
	local pPlayer = Players[iPlayer]
	local pPlot = Map.GetPlot(iCityX, iCityY)
	local pCity = pPlot:GetPlotCity()
	local iCity = pCity:GetID()
	if HasTrait(pPlayer, traitNanMadolID) and pPlayer:IsAlive() then
		for pAtollCoralPlot in Cunt15_AdjacentPlotIterator(pPlot) do
			ClaimAdjacentCoralAtoll(pAtollCoralPlot, iPlayer, iCity)
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerCityFounded.Add(Jar_NanMadol_AtollCoralCultureBomb2) end

function Jar_NanMadolAdjustments(playerID)
	local pPlayer = Players[playerID]
	if not HasTrait(pPlayer, traitNanMadolID) then return end 
	for pCity in pPlayer:Cities() do
		for i = 0, pCity:GetNumCityPlots() - 1, 1 do
			local pPlot = pCity:GetCityIndexPlot(i)
			if pPlot:GetOwner() == playerID then
				ClaimAdjacentCoralAtoll(pPlot, playerID, pCity:GetID())
			end
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_NanMadolAdjustments) end

local tSeaLuxury = {}
for row in DB.Query("SELECT a.ID ResID from Resources a, Improvement_ResourceTypes b, Improvements c where a.Type = b.ResourceType AND b.ImprovementType=c.Type AND c.Water=1 AND a.Happiness>0") do
	tSeaLuxury[row.ResID] = true
end
function Jar_NavalCanClaimSeaLuxury(unit, pPlayer) -- checks if a unit is on a valid plot
	local pBool = false;
	local pPlot = unit:GetPlot();
	if unit:MovesLeft() <= 1 then return pBool end
	if not pPlayer:IsHuman() then return pBool end
	if not HasTrait(pPlayer, traitNanMadolID) then return pBool end
	if not pPlayer:IsGoldenAge() then return pBool end
	if not unit:IsHasPromotion(iGAClaimSeaLuxPromo) then return pBool end
	if pPlot then
		if pPlot:GetOwner() ~= -1 then return pBool end
		if tSeaLuxury[pPlot:GetResourceType()] then pBool = true end
	end
	return pBool
end

function Jar_NavalDoClaimSeaLuxury(iPlayer, hUnit)
	local hPlayer = Players[iPlayer]
	local hTeam = hPlayer:GetTeam()
	local hUnitPlot = hUnit:GetPlot();
	local resID = hUnitPlot:GetResourceType()
	local pCity = Neirai_GetNearestCity(hPlayer, hUnitPlot)
	local iCity = pCity:GetID()
	hUnitPlot:SetOwner(iPlayer, iCity, true, true);
	hUnitPlot:SetRevealed(hTeam, true)
	hUnitPlot:SetImprovementType( GetImprovementIDFromResourceID(resID) )
	ClaimAdjacentCoralAtoll(hUnitPlot, iPlayer, iCity)
	local pCapital = hPlayer:GetCapitalCity()
	local bonusFood = math.floor(pCapital:GetYieldRate(YieldTypes.YIELD_FOOD) *FactorProdFoodClaimedSeaLux)
	pCapital:ChangeFood(bonusFood)
	local bonusProduction = math.floor(pCapital:GetYieldRate(YieldTypes.YIELD_PRODUCTION) *FactorProdFoodClaimedSeaLux)
	pCapital:ChangeProduction(bonusProduction)
	hUnit:SetHasPromotion(iGAClaimSeaLuxPromo, false)
	-- Notification
	local sTitle = "A Naval Unit claimed a Sea Luxury Resource!"
	local sText = "+1 " .. GameInfo.Resources[resID].IconString .. " " .. Locale.ConvertTextKey( GameInfo.Resources[resID].Description ) .. ". +" .. bonusFood .. " [ICON_FOOD], +" .. bonusProduction .. " [ICON_PRODUCTION] in [ICON_CAPITAL] " .. pCapital:GetName() .. "."
	hPlayer:AddNotification(NotificationTypes.NOTIFICATION_DISCOVERED_LUXURY_RESOURCE, sText, sTitle, hUnitPlot:GetX(), hUnitPlot:GetY(), resID)
end

function Jar_NavalDoClaimSeaLuxuryAI(playerID)
	local aPlayer = Players[playerID]
	local aTeam = Teams[aPlayer:GetTeam()]
	if not aPlayer:IsAlive() then return end
	if not HasTrait(aPlayer, traitNanMadolID) then return end 
	if not aPlayer:IsGoldenAge() then return end
	if aPlayer:GetNumUnitPromotions(iGAClaimSeaLuxPromo) <1 then return end
	for aUnit in aPlayer:Units() do
		local aUnitPlot = aUnit:GetPlot();
		if aUnit:IsHasPromotion(iGAClaimSeaLuxPromo) and aUnitPlot then
			local bcheck1 = not aPlayer:IsHuman()
			local bcheck2 = aPlayer:IsHuman() and aUnit:IsAutomated()
			if bcheck1 or bcheck2 then
				if aUnit:IsTrade() then GAMissionPlotRadiusCheckedByAI = 1 end
				local bcheck3 = true
				for adjacentPlot in PlotAreaSweepIterator(aUnitPlot, GAMissionPlotRadiusCheckedByAI, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
					local resID = adjacentPlot:GetResourceType()
					if tSeaLuxury[resID] and adjacentPlot:GetOwner() == -1 then
						local pCity = Neirai_GetNearestCity(aPlayer, adjacentPlot)
						local iCity = pCity:GetID()
						adjacentPlot:SetOwner(playerID, iCity, true, true);
						adjacentPlot:SetRevealed(aTeam, true)
						adjacentPlot:SetImprovementType( GetImprovementIDFromResourceID( resID) )
						ClaimAdjacentCoralAtoll(adjacentPlot, playerID, iCity)
						local pCapital = aPlayer:GetCapitalCity()
						local bonusFood = math.floor(pCapital:GetYieldRate(YieldTypes.YIELD_FOOD) *FactorProdFoodClaimedSeaLux)
						pCapital:ChangeFood(bonusFood)
						local bonusProduction = math.floor(pCapital:GetYieldRate(YieldTypes.YIELD_PRODUCTION) *FactorProdFoodClaimedSeaLux)
						pCapital:ChangeProduction(bonusProduction)
						bcheck3 = false
						if aPlayer:IsHuman() then
							-- Notification
							local sTitle = "A Naval Unit claimed a Sea Luxury Resource!"
							local sText = "+1 " .. GameInfo.Resources[resID].IconString .. " " .. Locale.ConvertTextKey( GameInfo.Resources[resID].Description ) .. ". +" .. bonusFood .. " [ICON_FOOD], +" .. bonusProduction .. " [ICON_PRODUCTION] in [ICON_CAPITAL] " .. pCapital:GetName() .. "."
							aPlayer:AddNotification(NotificationTypes.NOTIFICATION_DISCOVERED_LUXURY_RESOURCE, sText, sTitle, adjacentPlot:GetX(), adjacentPlot:GetY(), resID)
						end
					end
				end
				aUnit:SetHasPromotion(iGAClaimSeaLuxPromo, bcheck3)
			end
		end
		GAMissionPlotRadiusCheckedByAI = 2
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_NavalDoClaimSeaLuxuryAI) end

-- Jar_NanMadolGAship_CustomMissionStart
local function Jar_NanMadolGAship_CustomMissionStart(playerID, unitID, missionID, data1, data2, flags, turn)
	local pPlayer = Players[playerID]
	local unit = pPlayer:GetUnitByID(unitID)
	local plot = unit:GetPlot()
	if missionID == missionGAClaimSeaLux then
		Jar_NavalDoClaimSeaLuxury(playerID, unit)
		return CUSTOM_MISSION_ACTION
	end
	return CUSTOM_MISSION_NO_ACTION
end

-- Jar_NanMadolGAship_CustomMissionCompleted
local function Jar_NanMadolGAship_CustomMissionCompleted(playerID, unitID, missionID, data1, data2, flags, turn)
	return (missionID == missionGAClaimSeaLux)
end

-- Jar_NanMadolGAship_CustomMissionPossible
local function Jar_NanMadolGAship_CustomMissionPossible(playerID, unitID, missionID, data1, data2, _, _, plotX, plotY, bTestVisible)
	if ((not playerID) or (not unitID) or (not missionID)) then return end
	local pPlayer = Players[playerID]
	local unit = pPlayer:GetUnitByID(unitID)
	if missionID == missionGAClaimSeaLux then
		return Jar_NavalCanClaimSeaLuxury(unit, pPlayer)
	end
	return false
end

GameEvents.CustomMissionStart.Add(Jar_NanMadolGAship_CustomMissionStart)
GameEvents.CustomMissionCompleted.Add(Jar_NanMadolGAship_CustomMissionCompleted)
GameEvents.CustomMissionPossible.Add(Jar_NanMadolGAship_CustomMissionPossible)


function Jar_PlaceAtoll(pPlot, iPlayer, iCity, pTeam, minradius, maxradius, bKepidau)
	local bdone= false
	for radius=minradius,maxradius do
		if not bdone then
			for adjacentPlot in PlotAreaSweepIterator(pPlot, radius, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_EXCLUDE) do
				if adjacentPlot and adjacentPlot:GetFeatureType() == -1 and adjacentPlot:GetResourceType() == -1 and adjacentPlot:GetTerrainType() == iCoast and not adjacentPlot:IsLake() then
					if adjacentPlot:GetOwner() == -1 or adjacentPlot:GetOwner() == iPlayer then
						if Players[iPlayer]:IsHuman() then
							newUnit = Players[iPlayer]:InitUnit(GameInfoTypes.UNIT_JAR_ATOLLER, adjacentPlot:GetX(), adjacentPlot:GetY());
							newUnit:PushMission(GameInfoTypes.MISSION_BUILD, GameInfoTypes.BUILD_JAR_DUMMY_ATOLL, -1, 0, 0, 1)
							--adjacentPlot:SetImprovementType(-1)
							if bKepidau then
								-- Notification
								local sTitle = "Atoll from Kepidau construction!"
								local sText = "A beautiful Atoll appeared in " .. pCity:GetName()
								Players[iPlayer]:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, sText, sTitle, adjacentPlot:GetX(), adjacentPlot:GetY())
							end
						else adjacentPlot:SetFeatureType(fAtollID)
						end
						if adjacentPlot:GetOwner() == iPlayer then
							for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
								local otherPlot = Map.PlotDirection(adjacentPlot:GetX(), adjacentPlot:GetY(), direction)
								if otherPlot and otherPlot:GetOwner() ~= iPlayer then
									otherPlot:SetOwner(iPlayer, iCity, true, true);
									otherPlot:SetRevealed(pTeam, true)
								end
							end	
						end
						bdone= true
						break
					end
				end
			end
		end
	end
end

function Jar_NanMadolAtollStart(iPlayer, iCityX, iCityY)
	local pPlayer = Players[iPlayer]
	if not HasTrait(pPlayer, traitNanMadolID) then return end
	if not pPlayer:IsAlive() then return end
	local pTeam = pPlayer:GetTeam()
	local pPlot = Map.GetPlot(iCityX, iCityY)
	local pCity = pPlot:GetPlotCity()
	local iCity = pCity:GetID()
	if pCity:IsCapital() then
		local bNoAtolls = true
		for adjacentPlot in PlotAreaSweepIterator(pPlot, CityWorkingRadius, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_EXCLUDE) do
			if adjacentPlot and adjacentPlot:GetFeatureType() == fAtollID then
				bNoAtolls = false
				break
			end
		end
		if bNoAtolls then
			Jar_PlaceAtoll(pPlot, iPlayer, iCity, pTeam, 1, CityWorkingRadius, false)
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerCityFounded.Add(Jar_NanMadolAtollStart) end
--========================================================================================================================
-- NAN MADOL UB: Kepidau
--========================================================================================================================
tBuildingFreeHarbor = {}
for row in DB.Query("select ID From Buildings where FreeBuildingThisCity='BUILDINGCLASS_HARBOR'") do
	tBuildingFreeHarbor[row.ID] = true
end

function Jar_KepidauAtollSpawn(iPlayer, iCity, iBuildingType, bGold, bFaithOrCulture)
	local pPlayer = Players[iPlayer];
	local pTeam = pPlayer:GetTeam()
	local pCity = pPlayer:GetCityByID(iCity)
	local pPlot = pCity:Plot()
	if not HasTrait(pPlayer, traitNanMadolID) then return end 
	if not pPlayer:IsAlive() then return end
	if iBuildingType == iKepidau or tBuildingFreeHarbor[iBuildingType] then
		Jar_PlaceAtoll(pPlot, iPlayer, iCity, pTeam, 2, CityWorkingRadius, true)
	end
end
if g_IsTraitActive then GameEvents.CityConstructed.Add(Jar_KepidauAtollSpawn) end

function Jar_KepidauAirliftGoldenAge(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:CountNumBuildings(iKepidau) <1 then return end
	if pPlayer:CountNumBuildings(iKepidau) >1 and bKepidauAirliftEnabled then
		for pCity in pPlayer:Cities() do
			if pCity:IsHasBuilding(iKepidau) then
				if pPlayer:IsGoldenAge() and not pCity:IsHasBuilding(iKepidauAirlift) then
					pCity:SetNumRealBuilding(iKepidauAirlift, 1)
				elseif (not pPlayer:IsGoldenAge()) and pCity:IsHasBuilding(iKepidauAirlift) then
					pCity:SetNumRealBuilding(iKepidauAirlift, 0)
				else
				end
			end
		end
	end
	if not bKepidauAirliftEnabled then
		for pUnit in pPlayer:Units() do
			local unitplot = pUnit:GetPlot()
			if not pUnit:IsHasPromotion(iKepidauPromo) then
				if unitplot:IsCity() then
					local pCity = unitplot:GetPlotCity()
					if pCity:IsHasBuilding(iKepidau) then
						pUnit:SetHasPromotion(iKepidauPromo, true)
					end
				else
					for adjacentPlot in Cunt15_AdjacentPlotIterator(unitplot) do
						if adjacentPlot and adjacentPlot:IsCity() then
							local pCity = adjacentPlot:GetPlotCity()
							if pCity:IsHasBuilding(iKepidau) then
								pUnit:SetHasPromotion(iKepidauPromo, true)
								break
							end
						end
					end
				end
			end
			if pUnit:IsHasPromotion(iKepidauPromo) then
				local bcheck = pUnit:IsEmbarked() and unitplot:IsWater()
				if pPlayer:IsGoldenAge() then
					pUnit:SetHasPromotion(iKepidauPromo3, bcheck)
					pUnit:SetHasPromotion(iKepidauPromo2, false)	
				else
					pUnit:SetHasPromotion(iKepidauPromo2, bcheck)
					pUnit:SetHasPromotion(iKepidauPromo3, false)	
				end
			end		
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_KepidauAirliftGoldenAge) end

function Jar_KepidauEmbarkedMovement(iPlayer, iUnitID)
	local pPlayer = Players[iPlayer]
	local pUnit = pPlayer:GetUnitByID(iUnitID)
	local unitplot = pUnit:GetPlot()
	if not pPlayer:IsAlive() then return end
	if not unitplot then return end
	if not pUnit:IsHasPromotion(iKepidauPromo) then
		if pPlayer:CountNumBuildings(iKepidau) >0 then
			if unitplot:IsCity() then
				local pCity = unitplot:GetPlotCity()
				if pCity:IsHasBuilding(iKepidau) then
					pUnit:SetHasPromotion(iKepidauPromo, true)
				end
			else
				for adjacentPlot in Cunt15_AdjacentPlotIterator(unitplot) do
					if adjacentPlot and adjacentPlot:IsCity() then
						local pCity = adjacentPlot:GetPlotCity()
						if pCity:IsHasBuilding(iKepidau) then
							pUnit:SetHasPromotion(iKepidauPromo, true)
							break
						end
					end
				end
			end
		end
	end
	if pUnit:IsHasPromotion(iKepidauPromo) then
		local bcheck = pUnit:IsEmbarked() and unitplot:IsWater()
		if pPlayer:IsGoldenAge() then
			pUnit:SetHasPromotion(iKepidauPromo3, bcheck)
			pUnit:SetHasPromotion(iKepidauPromo2, false)	
		else
			pUnit:SetHasPromotion(iKepidauPromo2, bcheck)
			pUnit:SetHasPromotion(iKepidauPromo3, false)	
		end
		--pUnit:ChangeMoves(movechange *GameDefines.MOVE_DENOMINATOR)
	end
end
if g_IsTraitActive and not bKepidauAirliftEnabled then GameEvents.UnitSetXY.Add(Jar_KepidauEmbarkedMovement) end

local tUnitHarborOnFound = {}
for row in DB.Query("select a.ID UnitID from Units a, Unit_BuildOnFound b where a.Type = b.UnitType and b.BuildingClassType='BUILDINGCLASS_HARBOR'") do
	tUnitHarborOnFound[row.UnitID] = true
end
function Jar_Kepidau_Unit_BuildOnFound(iPlayer, iUnit, iUnitType, iPlotX, iPlotY)
	local pPlayer = Players[iPlayer]
	local pTeam = pPlayer:GetTeam()
	local pPlot = Map.GetPlot(iPlotX, iPlotY)
	local pCity = pPlot:GetPlotCity()
	if not pPlot then return end
	if not HasTrait(pPlayer, traitNanMadolID) then return end
	if tUnitHarborOnFound[iUnitType] then
		if pPlayer:IsGoldenAge() and bKepidauAirliftEnabled then
			pCity:SetNumRealBuilding(iKepidauAirlift, 1)
		end
		Jar_PlaceAtoll(pPlot, iPlayer, pCity:GetID(), pTeam, 2, CityWorkingRadius, true)
	end
end
if g_IsTraitActive then GameEvents.UnitCityFounded.Add(Jar_Kepidau_Unit_BuildOnFound) end
--========================================================================================================================
-- NAN MADOL UU: Nahnken
--========================================================================================================================
function Jar_ProductionFromKillNearGG(iAttackingPlayer, iAttackingUnit, attackerDamage, attackerFinalDamage, attackerMaxHP, iDefendingPlayer, iDefendingUnit, defenderDamage, defenderFinalDamage, defenderMaxHP, iInterceptingPlayer, iInterceptingUnit, interceptorDamage, plotX, plotY)
	if iAttackingPlayer==-1 then return end
	local pAttPlayer = Players[iAttackingPlayer]
	local pDefPlayer = Players[iDefendingPlayer]
	local pDefUnit = pDefPlayer:GetUnitByID(iDefendingUnit)
	local pAttUnit = pAttPlayer:GetUnitByID(iAttackingUnit)
	local pDefendedPlot = Map.GetPlot(plotX, plotY)
	if pAttPlayer:GetNumUnitPromotions(iNahnkenPromo) <1 then return end
	if pAttPlayer and pDefUnit then -- Avoid the case the combat was aborted...
		local defenderHealth = defenderMaxHP - defenderFinalDamage
		if defenderHealth <= 0 then
			local pPlot = pAttUnit:GetPlot()
			local bcheck = false
			for adjacentPlot in PlotAreaSweepIterator(pPlot, 2, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_INWARDS, CENTRE_INCLUDE) do
				if adjacentPlot:IsUnit() then
					for pUnit in Cunt15_PlotUnitsIterator(adjacentPlot) do
						if pUnit:IsHasPromotion(iNahnkenPromo) then
							bcheck = true
							break
						end
					end
				end
			end
			if bcheck then
				local prodkill = math.floor( FactorProductionFromNearbyKill* math.max( pDefUnit:GetBaseCombatStrength(), pDefUnit:GetBaseRangedCombatStrength() ))
				local pCapital = pAttPlayer:GetCapitalCity()
				local pCity = Neirai_GetNearestCity(pAttPlayer, pPlot)
				if pCity == pCapital then
					pCapital:ChangeProduction(prodkill*2)
					if pAttPlayer:IsHuman() then
					-- Notification
						local sTitle = "Enslaved defeated enemy near Nahnken"
						local sText = "+ ".. prodkill*2 .. " [ICON_PRODUCTION] in [ICON_CAPITAL]" .. pCapital:GetName() .. "."
						pAttPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sText, sTitle, plotX, plotY)
					end
				else
					pCapital:ChangeProduction(prodkill)
					pCity:ChangeProduction(prodkill)
					if pAttPlayer:IsHuman() then
						-- Notification
						local sTitle = "Enslaved defeated enemy near Nahnken"
						local sText = "[ICON_BULLET]+ ".. prodkill .. " [ICON_PRODUCTION] in [ICON_CAPITAL]" .. pCapital:GetName() .. ". [NEWLINE][ICON_BULLET]+ ".. prodkill .. " [ICON_PRODUCTION] in " .. pCity:GetName() .. "."
						pAttPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sText, sTitle, plotX, plotY)
					end
				end
			end
		end
	end
end
if g_IsTraitActive then GameEvents.CombatResult.Add(Jar_ProductionFromKillNearGG) end

--[[
Jar_NahnkenAtoll_BuildFinished_PlayerID = 0
Jar_NahnkenAtoll_BuildFinished_X = 0
Jar_NahnkenAtoll_BuildFinished_Y = 0
Jar_NahnkenAtoll_BuildFinished_Improvement = 0
function Jar_GoldenAgePointsNahnkenAtoll(playerID, plotX, plotY, improvementID)
	if(Jar_NahnkenAtoll_BuildFinished_PlayerID == playerID and Jar_NahnkenAtoll_BuildFinished_X == plotX and Jar_NahnkenAtoll_BuildFinished_Y == plotY and Jar_NahnkenAtoll_BuildFinished_Improvement == improvementID) then
		return;
	end
	-- Store inputs for duplication check
	Jar_NahnkenAtoll_BuildFinished_PlayerID = playerID;	
	Jar_NahnkenAtoll_BuildFinished_X = plotX;					
	Jar_NahnkenAtoll_BuildFinished_Y = plotY;					
	Jar_NahnkenAtoll_BuildFinished_Improvement = improvementID;
	local pPlot = Map.GetPlot(plotX, plotY)
	local pCity = pPlot:GetWorkingCity()
	local pPlayer = Players[playerID]
	if pPlayer:IsAlive() then
		if improvementID == GameInfoTypes.IMPROVEMENT_JAR_NAHNKEN_IN or improvementID == GameInfoTypes.IMPROVEMENT_JAR_NAHNKEN_OUT then
			local bonus = math.floor(pPlayer:GetTotalPopulation()*(1+ pPlayer:GetCurrentEra())*NahnkenGAPfactor)
			pPlayer:ChangeGoldenAgeProgressMeter(bonus)
			if pPlayer:IsHuman() then
				local text = "[COLOR_GREAT_PEOPLE_STORED]+" .. bonus .. " [ICON_GOLDEN_AGE][ENDCOLOR]"
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(plotX, plotY))), text, 2)
			end
		end
	end
end
if g_IsTraitActive then GameEvents.BuildFinished.Add(Jar_GoldenAgePointsNahnkenAtoll) end ]]

function Jar_GoldenAgePointsNahnkenAtoll(iPlayer, iUnit, iPlotX, iPlotY, iBuild)
	local pPlayer = Players[iPlayer]
	local pPlot = Map.GetPlot(iPlotX, iPlotY)
	if not pPlayer:IsAlive() then return end
	if not pPlot then return end
	local pUnit = pPlayer:GetUnitByID(iUnit)
	if not pUnit then return end
	if pUnit:IsHasPromotion(iNahnkenPromo) then
		local bonus = math.floor(pPlayer:GetTotalPopulation()*(1+ pPlayer:GetCurrentEra())*NahnkenGAPfactor)
		pPlayer:ChangeGoldenAgeProgressMeter(bonus)
		if iBuild == GameInfoTypes.BUILD_JAR_NAHNKEN_IN or iBuild == GameInfoTypes.BUILD_JAR_NAHNKEN_OUT then
			pPlot:SetImprovementType(-1)
		end
		if pPlayer:IsHuman() then
			local text = "[COLOR_GREAT_PEOPLE_STORED]+" .. bonus .. " [ICON_GOLDEN_AGE][ENDCOLOR]"
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(plotX, plotY))), text, 2)
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerBuilt.Add(Jar_GoldenAgePointsNahnkenAtoll) end
--=======================================================================================================================
--MORE UNIQUE COMPONENTS
--=======================================================================================================================
local resOil = GameInfoTypes.RESOURCE_OIL
local iProaPromo = GameInfoTypes.PROMOTION_JAR_PROA
local iRebbelibPromo = GameInfoTypes.PROMOTION_JAR_PACIFARER
local iNahs = GameInfoTypes.BUILDING_JAR_NAHS
local iNahsDummy = GameInfoTypes.BUILDING_JAR_NAHSDUMMY
--========================================================================================================================
-- NAN MADOL UB2: Nahs
--========================================================================================================================
function Jar_KepidauAirliftGoldenAge(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:CountNumBuildings(iNahs) <1 then return end
	for pCity in pPlayer:Cities() do
		local pPlot = pCity:Plot()
		if pCity:IsHasBuilding(iNahs) then
			local bcheck = false
			for adjacentPlot in PlotAreaSweepIterator(pPlot, CityWorkingRadius, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_INWARDS, CENTRE_INCLUDE) do
				if adjacentPlot:IsUnit() then
					for pUnit in Cunt15_PlotUnitsIterator(adjacentPlot) do
						if pUnit:IsHasPromotion(iNahnkenPromo) then
							bcheck = true
							break
						end
					end
				end
				if bcheck then break
				elseif (adjacentPlot:GetImprovementType() == GameInfoTypes.IMPROVEMENT_CITADEL or adjacentPlot:GetImprovementType() == GameInfoTypes.IMPROVEMENT_CITADEL_JAR_ATOLL) then
					bcheck = true
					break
				end
			end
			if bcheck then pCity:SetNumRealBuilding(iNahsDummy, 1)
			else pCity:SetNumRealBuilding(iNahsDummy, 0)
			end
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_KepidauAirliftGoldenAge) end
--========================================================================================================================
-- NAN MADOL UU2: Proa
--========================================================================================================================
local techBio;
for row in DB.Query("SELECT TechReveal FROM Resources WHERE Type = 'RESOURCE_OIL'") do
	techBio = row.TechReveal
end

function Jar_ProaAdjSeaResHeal(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:GetNumUnitPromotions(iProaPromo) <1 then return end
	for pUnit in pPlayer:Units() do
		if pUnit:IsHasPromotion(iProaPromo) then
			local pUnitPlot = pUnit:GetPlot()
			for adjacentPlot in Cunt15_AdjacentPlotIterator(pUnitPlot) do
				if adjacentPlot and adjacentPlot:IsWater() then
					local iOwnerID = adjacentPlot:GetOwner()
					if iOwnerID == playerID or iOwnerID == -1 then
						if adjacentPlot:GetResourceType() > -1 then
							if adjacentPlot:GetResourceType() ~= resOil then								
								pUnit:ChangeDamage(-10)
								break
							elseif Teams[pPlayer:GetTeam()]:IsHasTech(techBio) then
								pUnit:ChangeDamage(-10)
								break
							end
						elseif tFeatures[adjacentPlot:GetFeatureType()] then
							pUnit:ChangeDamage(-10)
							break
						end
					end
				end
			end
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_ProaAdjSeaResHeal) end
--=======================================================================================================================
-- Fix embarked worked visual glitch
local tImproAllowsWalkWater= {}
for row in DB.Query("select b.ID BuildID, b.Type BuildType from Improvements a, Builds b where a.AllowsWalkWater=1 and a.Type=b.ImprovementType and b.Kill=0") do
	tImproAllowsWalkWater[row.BuildID] = true
end
function Jar_BuildEmbarkedFix(iPlayer, iUnit, iPlotX, iPlotY, iBuild)
	local pPlayer = Players[iPlayer]
	local pPlot = Map.GetPlot(iPlotX, iPlotY)
	if not pPlayer:IsAlive() then return end
	if not pPlot then return end
	local pUnit = pPlayer:GetUnitByID(iUnit)
	if not pUnit then return end
	if tImproAllowsWalkWater[iBuild] then
		pUnit:SetEmbarked(false)
		local iTeam = pPlayer:GetTeam()
		local pTeam = Teams[iTeam]
		pTeam:UpdateEmbarkGraphics()
	end
end
GameEvents.PlayerBuilt.Add(Jar_BuildEmbarkedFix)
--------------------------------------------------------------
print("Nan Madol functions loaded!")
--=======================================================================================================================
--=======================================================================================================================